'use strict';

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = defaultImportsChecker;
require('colors');

var modules = {};
var namespaceImports = [];

function validateModules() {
  var errors = [];
  Object.keys(modules).map(function (k) {
    return modules[k];
  }).forEach(function (module) {
    module.dependencies.forEach(function (dep) {
      var depModule = modules[dep.filename];
      if (depModule) {
        if (!depModule.exportDefault) {
          var message = 'ERROR : module \'' + module.name + '\' try to use default binding\nof module \'' + depModule.name + '\' at line [' + dep.line + '] but there is none.\n | import ' + dep.variable + ' from \'' + dep.path + '\';\nYou should add a default binding to \'' + depModule.name + '\'\nor use the following import :\n | import * as ' + dep.variable + ' from  \'' + dep.path + '\';';
          if (errors.indexOf(message) < 0) {
            errors.push(message);
          }
        }
      }
    });
  });
  var warnings = [];
  namespaceImports.forEach(function (imprt) {
    var dep = modules[imprt.filename];
    if (dep && dep.exportDefault && !dep.exportOther) {
      var message = 'WARNING : module \'' + imprt.from + '\' is using namespace import of module \'' + imprt.filename + '\'\nat line [' + imprt.line + '] but this module has default export. Maybe it\'s an error.\n | import * as ' + imprt.variable + ' from \'' + imprt.path + '\';';
      if (errors.indexOf(message) < 0) {
        warnings.push(message);
      }
    }
  });
  return [errors, warnings];
}

function defaultImportsChecker() {
  var id = undefined;
  return {
    visitor: {
      Program: {
        enter: function enter(node, parent) {
          // scope
          clearTimeout(id);
          modules[parent.file.opts.filename] = {
            name: parent.file.opts.filename,
            exportDefault: false,
            exportOther: false,
            dependencies: []
          };
        },
        exit: function exit() {
          // node, parent, scope
          id = setTimeout(function () {
            clearTimeout(id);

            var _validateModules = validateModules();

            var _validateModules2 = _slicedToArray(_validateModules, 2);

            var errors = _validateModules2[0];
            var warnings = _validateModules2[1];

            if (warnings.length > 0) {
              console.log(warnings.join('\n\n').yellow);
            }
            if (warnings.length > 0 && errors.length > 0) {
              console.log('');
            }
            if (errors.length > 0) {
              console.log(errors.join('\n\n').red);
            }
          }, 100);
        }
      },
      ExportDefaultDeclaration: function ExportDefaultDeclaration(node, parent) {
        // scope
        // filename : parent.file.opts.filename
        modules[parent.file.opts.filename].exportDefault = true;
      },
      ExportDeclaration: function ExportDeclaration(node, parent) {
        // scope
        // filename : parent.file.opts.filename
        if (String(node.type) !== 'ExportDefaultDeclaration') {
          modules[parent.file.opts.filename].exportOther = true;
        }
      },
      ImportDeclaration: function ImportDeclaration(path, state) {
        // current file : state.file.opts.filename
        path.node.specifiers.map(function (specifier) {
          // variable name : specifier.local.name
          // relative module : path.node.source.value

          // ImportSpecifier => import { Foo } from './bar';
          // ImportDefaultSpecifier => import Foo from './bar';
          // ImportNamespaceSpecifier => import * as Foo from './bar';

          function extractFilename() {
            var relativePath = path.node.source.value.split('\/');
            var filePath = state.file.opts.filename.split('\/');
            filePath.pop();
            for (var i in relativePath) {
              var part = relativePath[i];
              if (part === '.') {
                // nothing
              } else if (part === '..') {
                  filePath.pop();
                } else {
                  filePath.push(part);
                }
            }
            return filePath.join('/') + '.js';
          }

          if (String(specifier.type) === 'ImportNamespaceSpecifier' && path.node.source.value.startsWith('.')) {
            var filename = extractFilename();
            namespaceImports.push({
              from: state.file.opts.filename,
              filename: filename,
              variable: specifier.local.name,
              path: path.node.source.value,
              line: specifier.loc.start.line
            });
          }
          if (String(specifier.type) === 'ImportDefaultSpecifier' && path.node.source.value.startsWith('.')) {
            var filename = extractFilename();
            modules[state.file.opts.filename].dependencies.push({
              from: state.file.opts.filename,
              filename: filename,
              variable: specifier.local.name,
              path: path.node.source.value,
              line: specifier.loc.start.line
            });
          }
        });
      }
    }
  };
}